@@ -1,5 +1,4 @@
 #![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]
-#![feature(const_fn)]
 
 use glium::{
     glutin,
@@ -14,7 +13,7 @@ use image::{ImageBuffer, Rgba};
 use imgui::{Context, FontConfig, FontSource};
 use imgui_glium_renderer::Renderer;
 use imgui_winit_support::{HiDpiMode, WinitPlatform};
-use std::{env, time::Instant};
+use std::{env, path::PathBuf, time::Instant};
 
 mod app;
 mod clipboard;
@@ -27,7 +26,7 @@ mod icon;
 mod image_view;
 
 pub enum UserEvent {
-    ImageLoaded(ImageBuffer<Rgba<u16>, Vec<u16>>),
+    ImageLoaded(ImageBuffer<Rgba<u16>, Vec<u16>>, PathBuf),
     ImageError(String),
 }
 
@@ -77,12 +76,24 @@ impl System {
 
         let hidpi_factor = platform.hidpi_factor();
         let font_size = (13.0 * hidpi_factor) as f32;
-        imgui.fonts().add_font(&[FontSource::DefaultFontData {
-            config: Some(FontConfig {
+        //let font_size = (18.0 * hidpi_factor) as f32;
+        imgui.fonts().add_font(&[
+            /*FontSource::TtfData {
+                data: include_bytes!("../fonts/segoeui.ttf"),
                 size_pixels: font_size,
-                ..FontConfig::default()
-            }),
-        }]);
+                config: Some(FontConfig {
+                    rasterizer_multiply: 1.0,
+                    glyph_ranges: imgui::FontGlyphRanges::default(),
+                    ..FontConfig::default()
+                }),
+            },*/
+            FontSource::DefaultFontData {
+                config: Some(FontConfig {
+                    size_pixels: font_size,
+                    ..FontConfig::default()
+                }),
+            },
+        ]);
         imgui.io_mut().font_global_scale = (1.0 / hidpi_factor) as f32;
 
         let renderer = Renderer::init(&mut imgui, &display).expect("Failed to initialize renderer");
